@@ -8,12 +8,15 @@
 #' @param file        a character string indicating the name of the Mplus input
 #'                    file with or without the file extension \code{.inp}, e.g.,
 #'                    \code{"Mplus_Input.inp"} or \code{"Mplus_Input"}.
-#' @param replace.inp logical: if \code{TRUE} (default), an existing input
-#'                    file will be replaced.
 #' @param data        a matrix or data frame from which the variables names for
 #'                    the subsection \code{NAMES} are extracted when using the
-#'                    \code{...} specification in the \code{VARIABLE} section.
-#' @param mplus.run  logical: if \code{TRUE}, the input file specified in the
+#'                    \code{...;} specification in the \code{VARIABLE} section.
+#' @param comment     logical: if \code{FALSE} (default), comments (i.e., text
+#'                    after the \code{!} symbol) are removed from the input text
+#'                    specified in the argument \code{x}.
+#' @param replace.inp logical: if \code{TRUE} (default), an existing input
+#'                    file will be replaced.
+#' @param mplus.run   logical: if \code{TRUE}, the input file specified in the
 #'                    argument \code{file} containing the input text specified
 #'                    in the argument \code{x} is run using the \code{mplus.run()}
 #'                    function.
@@ -28,16 +31,22 @@
 #'                    and \code{"modified"}, which only runs a model if the
 #'                    modified date for the input file is more recent than the
 #'                    output file modified date.
+#' @param Mplus       a character string for specifying the name or path of the
+#'                    Mplus executable to be used for running models. This covers
+#'                    situations where Mplus is not in the system's path, or where
+#'                    one wants to test different versions of the Mplus program.
+#'                    Note that there is no need to specify this argument for most
+#'                    users since it has intelligent defaults.
 #' @param print       a character vector indicating which results to show, i.e.
 #'                    \code{"all"} (default) for all results \code{"input"} for
 #'                    input command sections, and \code{"result"} for result sections.
-#' @param input       a character vector specifiying Mplus input command sections
+#' @param input       a character vector specifying Mplus input command sections
 #'                    included in the output (see 'Details' in the \code{\link{mplus.print}}
 #'                    function).
-#' @param result      a character vector specifiying Mplus result sections included
+#' @param result      a character vector specifying Mplus result sections included
 #'                    in the output (see 'Details' in the \code{\link{mplus.print}}
 #'                    function).
-#' @param exclude     a character vector specifiying Mplus input command or result
+#' @param exclude     a character vector specifying Mplus input command or result
 #'                    sections excluded from the output (see 'Details' in the
 #'                    \code{\link{mplus.print}} function).
 #' @param variable    logical: if \code{TRUE}, names of the variables in the data
@@ -66,20 +75,22 @@
 #' \describe{
 #' \item{\strong{\code{NAMES} Option}}{in the \code{VARIABLE} section used to
 #' assign names to the variables in the data set can be specified by using
-#' \code{...} and the \code{data} argument:
+#' the \code{...;} specification and the \code{data} argument:
 #'    \itemize{
 #'       \item{\code{Write Mplus Data File}}: In the first step, the Mplus data
 #'       file is written by using the \code{write.mplus()} function, e.g.
 #'       \code{write.mplus(ex3_1, file = "ex3_1.dat")}.
 #'       \item{\code{Specify Mplus Input}}: In the second step, the Mplus input
 #'       is specified as a character string. The \code{NAMES} option can be
-#'       specified by using \code{...}, e.g.,
-#'       \code{input <- 'DATA:     FILE IS ex3_1.dat;\nVARIABLE: ...\nMODEL:    y1 ON x1 x3;'}.
+#'       specified by using \code{...;}, e.g.,
+#'       \code{input <- 'DATA:     FILE IS ex3_1.dat;\nVARIABLE: ...;\nMODEL:    y1 ON x1 x3;'}.
 #'       \item{\code{Run Mplus Input}}: In the third step, the Mplus input is run
-#'       by using the \code{mplus()} function. Note that the argument \code{data}
-#'       needs to be specified given that the \code{NAMES} option was specified
-#'       by using \code{...} in the previous step, e.g.,
-#'       \code{mplus(input, file = "ex3_1.inp", data = ex3_1)}
+#'       by using the \code{mplus()} function. The argument \code{data} needs to
+#'       be specified given that the \code{NAMES} option was specified by using
+#'       \code{...;} in the previous step, e.g., \code{mplus(input, file = "ex3_1.inp", data = ex3_1)}.
+#'       Note that \code{...;} including the semicolon \code{;} needs to be specified,
+#'       i.e., \code{...} without the semicolon \code{;} will result in an error
+#'       message.
 #'    }
 #' }
 #' }
@@ -88,8 +99,9 @@
 #' Takuya Yanagida
 #'
 #' @seealso
-#' \code{\link{read.mplus}}, \code{\link{write.mplus}}, \code{\link{mplus.print}},
-#' \code{\link{mplus.update}}, \code{\link{mplus.run}}, \code{\link{mplus.lca}}
+#' \code{\link{read.mplus}}, \code{\link{write.mplus}}, \code{\link{mplus.update}},
+#' \code{\link{mplus.print}}, \code{\link{mplus.plot}}, \code{\link{mplus.bayes}},
+#' \code{\link{mplus.run}}, \code{\link{mplus.lca}}
 #'
 #' @references
 #' Muthen, L. K., & Muthen, B. O. (1998-2017). \emph{Mplus User's Guide} (8th ed.).
@@ -106,7 +118,7 @@
 #' \item{\code{input}}{list with input command sections}
 #' \item{\code{write}}{write command sections}
 #' \item{\code{result}}{list with input command sections (\code{input}) and
-#'                      result sections}
+#'                      result sections (\code{result})}
 #'
 #' @export
 #'
@@ -130,12 +142,12 @@
 #' mplus(input1, file = "ex3_1.inp")
 #'
 #' #----------------------------------------------------------------------------
-#' # Example 2: Alterantive specification using ... and the data argument
+#' # Example 2: Alternative specification using ...; and the data argument
 #'
 #' # Specify Mplus input, specify NAMES option by using ...
 #' input2 <- '
 #' DATA:     FILE IS ex3_1.dat;
-#' VARIABLE: ...
+#' VARIABLE: ...;
 #' MODEL:    y1 ON x1 x3;
 #' OUTPUT:   SAMPSTAT;
 #' '
@@ -143,9 +155,9 @@
 #' # Run Mplus input
 #' mplus(input2, file = "ex3_1.inp", data = ex3_1)
 #' }
-mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL, mplus.run = TRUE,
-                  show.out = FALSE, replace.out = c("always", "never", "modified"),
-                  print = c("all", "input", "result"),
+mplus <- function(x, file = "Mplus_Input.inp", data = NULL, comment = FALSE, replace.inp = TRUE,
+                  mplus.run = TRUE, show.out = FALSE, replace.out = c("always", "never", "modified"),
+                  Mplus = detect.mplus(), print = c("all", "input", "result"),
                   input = c("all", "default", "data", "variable", "define", "analysis", "model",
                             "montecarlo", "mod.pop", "mod.cov", "mod.miss", "message"),
                   result = c("all", "default", "summary.analysis.short", "summary.data.short",
@@ -189,19 +201,23 @@ mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL,
     # Check input 'x'
     if (isTRUE(grepl("...", x, fixed = TRUE))) {
 
+      # Semicolon included
+      unlist(strsplit(x, ""))[as.numeric(gregexec("\\.\\.\\.", x)[[1L]]) + 3L] |>
+        (\(z) if (isTRUE(z != ";" || is.na(z))) { stop("Please include the semicolon ; when using the \"...;\" specification.", call. = FALSE)} )()
+
       # Data argument specified
-      if (isTRUE(is.null(data))) { stop("Please specify a data frame or matrix for the argument 'data' when using the '...' specification.", call. = FALSE) }
+      if (isTRUE(is.null(data))) { stop("Please specify a data frame or matrix for the argument 'data' when using the '...;' specification.", call. = FALSE) }
 
       # NAMES specified
-      if (isTRUE(grepl("NAMES", toupper(x)))) { stop("Please do not specify the subection NAMES in the Mplus input text when using the '...' specification.") }
+      if (isTRUE(grepl("NAMES", toupper(x)))) { stop("Please do not specify the subection NAMES in the Mplus input text when using the '...;' specification.") }
 
     }
 
     # Check input 'file'
     if (isTRUE(!is.character(file) || length(file) != 1L)) { stop("Please specify a character string for the argument 'file'.", call. = FALSE) }
 
-    # Check input 'replace.inp'
-    if (isTRUE(!is.logical(replace.inp))) { stop("Please specify TRUE or FALSE for the argument 'replace.inp'.", call. = FALSE) }
+    # Check input 'comment'
+    if (isTRUE(!is.logical(comment))) { stop("Please specify TRUE or FALSE for the argument 'comment'.", call. = FALSE) }
 
     # Check input 'data'
     if (isTRUE(!is.null(data))) {
@@ -209,17 +225,22 @@ mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL,
       # Data frame or matrix for
       if (isTRUE(!is.data.frame(data) && !is.matrix(data))) { stop("Please specify a data frame or matrix for the argument 'data'.", call. = FALSE) }
 
-      # ... specification
-      if (isTRUE(!grepl("...", x, fixed = TRUE))) { stop("Please specify '...' in the Mplus input text when using the argument 'data'.", call. = FALSE) }
+      # ...; specification
+      if (isTRUE(!grepl("...;", x, fixed = TRUE))) { stop("Please specify '...;' in the Mplus input text when using the argument 'data'.", call. = FALSE) }
 
     }
 
+    # Check input 'replace.inp'
+    if (isTRUE(!is.logical(replace.inp))) { stop("Please specify TRUE or FALSE for the argument 'replace.inp'.", call. = FALSE) }
+
     # Check input 'mplus.run'
     if (isTRUE(!is.logical(mplus.run))) { stop("Please specify TRUE or FALSE for the argument 'mplus.run'.", call. = FALSE) }
 
     # Check input 'replace.out'
     if (isTRUE(!all(replace.out %in% c("always", "never", "modified")))) { stop("Character strings in the argument 'print' do not all match with \"always\", \"never\", or \"modified\".", call. = FALSE) }
 
+    if (isTRUE(!all(c("always", "never", "modified") %in% replace.out) && length(replace.out) != 1L)) { stop("Please specify a character string for the argument 'replace.out'", call. = FALSE) }
+
     # Check input 'print'
     if (isTRUE(!all(print %in% c("all", "input", "result")))) { stop("Character strings in the argument 'print' do not all match with \"all\", \"input\", or \"result\".", call. = FALSE) }
 
@@ -228,12 +249,12 @@ mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL,
     if (isTRUE(length(input.check) != 0L)) { stop(paste0(if (isTRUE(length(input.check) == 1L)) { "Character string " } else { "Character vector " }, "specified in the argument 'input' is not permissible: ", paste(dQuote(input.check), collapse = ", ")), call. = FALSE) }
 
     # Check input 'result'
-    result.check <- result[which(!result %in% c("all", "default", result.all))]
-    if (isTRUE(length(result.check) != 0L)) { stop(paste0(if (isTRUE(length(result.check) == 1L)) { "Character string " } else { "Character vector " }, "specified in the argument 'result' is not permissible: ", paste(dQuote(result.check), collapse = ", ")), call. = FALSE) }
+    result[which(!result %in% c("all", "default", c(input.all, result.all)))] |>
+      (\(z) if (isTRUE(length(z) != 0L)) { stop(paste0(if (isTRUE(length(z) == 1L)) { "Character string " } else { "Character vector " }, "specified in the argument 'result' is not permissible: ", paste(dQuote(z), collapse = ", ")), call. = FALSE) } )()
 
     # Check input 'exclude'
-    exclude.check <- exclude[which(!exclude %in% c(input.all, result.all))]
-    if (isTRUE(length(exclude.check) != 0L)) { stop(paste0(if (isTRUE(length(exclude.check) == 1L)) { "Character string " } else { "Character vector " }, "specified in the argument 'exclude' is not permissible: ", paste(dQuote(exclude.check), collapse = ", ")), call. = FALSE) }
+    exclude.check <- exclude[which(!exclude %in% result.all)] |>
+      (\(z) if (isTRUE(length(z) != 0L)) { stop(paste0(if (isTRUE(length(z) == 1L)) { "Character string " } else { "Character vector " }, "specified in the argument 'exclude' is not permissible: ", paste(dQuote(z), collapse = ", ")), call. = FALSE) })()
 
     # Check input 'variable'
     if (isTRUE(!is.logical(variable))) { stop("Please specify TRUE or FALSE for the argument 'variable'.", call. = FALSE) }
@@ -263,198 +284,226 @@ mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL,
   ## file Argument ####
 
   # File extension .inp
-  file <- ifelse(isTRUE(!grepl(".inp", file)), file <- paste0(file, ".inp"), file)
+  file <- ifelse(isTRUE(!grepl("\\.inp", file)), file <- paste0(file, ".inp"), file)
 
   # .out object
-  file.out <- sub(".inp", ".out", file)
+  file.out <- sub("\\.inp", ".out", file)
 
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## replace.out Argument ####
 
-  if (isTRUE(all(c("always", "never", "modified") %in% replace.out))) {
-
-    replace.out <- "always"
-
-  } else {
-
-    if (isTRUE(length(replace.out) != 1L)) { stop("Please specify a character string for the argument 'replace.out'", call. = FALSE) }
-
-  }
+  if (isTRUE(all(c("always", "never", "modified") %in% replace.out))) { replace.out <- "always" }
 
   #_____________________________________________________________________________
   #
   # Main Function --------------------------------------------------------------
 
+  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+  ## Remove Comments ####
+
+  if (isTRUE(!comment)) {
+
+    x <- paste(unlist(lapply(strsplit(unlist(strsplit(x, "\n")), ""), function(y) {
+
+      if (isTRUE(any(y == "!"))) {
+
+        if (which(misty::chr.omit(y, omit = " ", check = FALSE) == "!")[1L] == 1L) {
+
+          y <- NULL
+
+        } else {
+
+          y <- y[seq_along(y) < which(y == "!")[1L]]
+
+        }
+
+      }
+
+      return(if (isTRUE(!is.null(y))) paste(y, collapse = ""))
+
+    })), collapse = "\n")
+
+  }
+
+  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+  ## Upper Case Characters ####
+
+  x.upp <- toupper(x)
+
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## Extract Position of Input Command Sections ####
 
-  section.pos <- c(if (isTRUE(any(grepl("TITLE:", x)))) { as.numeric(gregexec("TITLE:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DATA:", x)))) { as.numeric(gregexec("DATA:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DATA IMPUTATION:", x)))) { as.numeric(gregexec("DATA IMPUTATION:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DATA WIDETOLONG:", x)))) { as.numeric(gregexec("DATA WIDETOLONG:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DATA LONGTOWIDE:", x)))) { as.numeric(gregexec("DATA LONGTOWIDE:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DATA TWOPART:", x)))) { as.numeric(gregexec("DATA TWOPART:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DATA MISSING:", x)))) { as.numeric(gregexec("DATA MISSING:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DATA SURVIVAL:", x)))) { as.numeric(gregexec("DATA SURVIVAL:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DATA COHORT:", x)))) { as.numeric(gregexec("DATA COHORT:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("VARIABLE:", x)))) { as.numeric(gregexec("VARIABLE:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("DEFINE:", x)))) { as.numeric(gregexec("DEFINE:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("ANALYSIS:", x)))) { as.numeric(gregexec("ANALYSIS:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("MODEL:", x)))) { as.numeric(gregexec("MODEL:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("MODEL INDIRECT:", x)))) { as.numeric(gregexec("MODEL INDIRECT:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("MODEL TEST:", x)))) { as.numeric(gregexec("MODEL TEST:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("MODEL PRIORS:", x)))) { as.numeric(gregexec("MODEL PRIORS:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("MONTECARLO:", x)))) { as.numeric(gregexec("MONTECARLO:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("MODEL POPULATION:", x)))) { as.numeric(gregexec("MODEL POPULATION:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("MODEL COVERAGE:", x)))) { as.numeric(gregexec("MODEL COVERAGE:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("MODEL MISSING:", x)))) { as.numeric(gregexec("MODEL MISSING:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("OUTPUT:", x)))) { as.numeric(gregexec("OUTPUT:", toupper(x))[[1L]]) } else { NULL },
-                   if (isTRUE(any(grepl("SAVEDATA:", x)))) { as.numeric(gregexec("SAVEDATA:", toupper(x))[[1L]]) } else { NULL })
+  section.pos <- c(if (isTRUE(any(grepl("TITLE:", x.upp)))) { as.numeric(gregexec("TITLE:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("DATA:", x.upp)))) {
+
+                     setdiff(as.numeric(gregexec("DATA:", x.upp)[[1L]]), as.numeric(gregexec("SAVEDATA:", x.upp)[[1L]]) + 4L)
+
+                   } else {
+
+                      NULL
+
+                   },
+                   if (isTRUE(any(grepl("DATA IMPUTATION:", x.upp)))) { as.numeric(gregexec("DATA IMPUTATION:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("DATA WIDETOLONG:", x.upp)))) { as.numeric(gregexec("DATA WIDETOLONG:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("DATA LONGTOWIDE:", x.upp)))) { as.numeric(gregexec("DATA LONGTOWIDE:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("DATA TWOPART:", x.upp)))) { as.numeric(gregexec("DATA TWOPART:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("DATA MISSING:", x.upp)))) { as.numeric(gregexec("DATA MISSING:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("DATA SURVIVAL:", x.upp)))) { as.numeric(gregexec("DATA SURVIVAL:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("DATA COHORT:", x.upp)))) { as.numeric(gregexec("DATA COHORT:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("VARIABLE:", x.upp)))) { as.numeric(gregexec("VARIABLE:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("DEFINE:", x.upp)))) { as.numeric(gregexec("DEFINE:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("ANALYSIS:", x.upp)))) { as.numeric(gregexec("ANALYSIS:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MODEL:", x.upp)))) { as.numeric(gregexec("MODEL:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MODEL INDIRECT:", x.upp)))) { as.numeric(gregexec("MODEL INDIRECT:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MODEL TEST:", x.upp)))) { as.numeric(gregexec("MODEL TEST:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MODEL PRIORS:", x.upp)))) { as.numeric(gregexec("MODEL PRIORS:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MONTECARLO:", x.upp)))) { as.numeric(gregexec("MONTECARLO:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MODEL POPULATION:", x.upp)))) { as.numeric(gregexec("MODEL POPULATION:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MODEL COVERAGE:", x.upp)))) { as.numeric(gregexec("MODEL COVERAGE:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MODEL MISSING:", x.upp)))) { as.numeric(gregexec("MODEL MISSING:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("MODEL CONSTRAINT:", x.upp)))) { as.numeric(gregexec("MODEL CONSTRAINT:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("PLOT:", x.upp)))) { as.numeric(gregexec("PLOT:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("OUTPUT:", x.upp)))) { as.numeric(gregexec("OUTPUT:", x.upp)[[1L]]) } else { NULL },
+                   if (isTRUE(any(grepl("SAVEDATA:", x.upp)))) { as.numeric(gregexec("SAVEDATA:", x.upp)[[1L]]) } else { NULL })
 
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## Extract Input Command Sections ####
 
-  title <- inpdata <- data.imp <- dat.wl <- data.lw <- data.tp <- data.miss <- data.surv <- data.coh <- inpvariable <- define <- analysis <- model <- mod.ind <- mod.test <- mod.prior <- montecarlo <- mod.pop <- mod.cov <- mod.miss <- inpoutput <- savedata <- plot <- NULL
+  title <- inpdata <- data.imp <- data.wl <- data.lw <- data.tp <- data.miss <- data.surv <- data.coh <- inpvariable <- define <- analysis <- model <- mod.ind <- mod.test <- mod.prior <- montecarlo <- mod.pop <- mod.cov <- mod.miss <- mod.const <- plot <- inpoutput <- savedata <- NULL
 
   #...................
   ### TITLE ####
 
-  if (isTRUE(any(grepl("TITLE:", x)))) { title <- .extract.section("TITLE:", x, section.pos) }
+  if (isTRUE(any(grepl("TITLE:", x.upp)))) { title <- .extract.section("TITLE:", x, section.pos) }
 
   #...................
   ### DATA ####
 
-  if (isTRUE(any(grepl("DATA:", x)))) { inpdata <- .extract.section("DATA:", x, section.pos) }
+  if (isTRUE(any(grepl("DATA:", x.upp)))) { inpdata <- .extract.section("DATA:", x, section.pos) }
 
   #...................
   ### DATA IMPUTATION ####
 
-  if (isTRUE(any(grepl("DATA IMPUTATION:", x)))) { data.imp <- .extract.section("DATA IMPUTATION:", x, section.pos) }
+  if (isTRUE(any(grepl("DATA IMPUTATION:", x.upp)))) { data.imp <- .extract.section("DATA IMPUTATION:", x, section.pos) }
 
   #...................
   ### DATA WIDETOLONG ####
 
-  if (isTRUE(any(grepl("DATA WIDETOLONG:", x)))) { data.wl <- .extract.section("DATA WIDETOLONG:", x, section.pos) }
+  if (isTRUE(any(grepl("DATA WIDETOLONG:", x.upp)))) { data.wl <- .extract.section("DATA WIDETOLONG:", x, section.pos) }
 
   #...................
   ### DATA LONGTOWIDE ####
 
-  if (isTRUE(any(grepl("DATA LONGTOWIDE:", x)))) { data.lw <- .extract.section("DATA LONGTOWIDE:", x, section.pos) }
+  if (isTRUE(any(grepl("DATA LONGTOWIDE:", x.upp)))) { data.lw <- .extract.section("DATA LONGTOWIDE:", x, section.pos) }
 
   #...................
   ### DATA TWOPART ####
 
-  if (isTRUE(any(grepl("DATA TWOPART:", x)))) { data.tp <- .extract.section("DATA TWOPART:", x, section.pos) }
+  if (isTRUE(any(grepl("DATA TWOPART:", x.upp)))) { data.tp <- .extract.section("DATA TWOPART:", x, section.pos) }
 
   #...................
   ### DATA MISSING ####
 
-  if (isTRUE(any(grepl("DATA MISSING:", x)))) { data.miss <- .extract.section("DATA MISSING:", x, section.pos) }
+  if (isTRUE(any(grepl("DATA MISSING:", x.upp)))) { data.miss <- .extract.section("DATA MISSING:", x, section.pos) }
 
   #...................
   ### DATA SURVIVAL ####
 
-  if (isTRUE(any(grepl("DATA SURVIVAL:", x)))) { data.surv <- .extract.section("DATA SURVIVAL:", x, section.pos) }
+  if (isTRUE(any(grepl("DATA SURVIVAL:", x.upp)))) { data.surv <- .extract.section("DATA SURVIVAL:", x, section.pos) }
 
   #...................
   ### DATA COHORT ####
 
-  if (isTRUE(any(grepl("DATA COHORT:", x)))) { data.coh <- .extract.section("DATA COHORT:", x, section.pos) }
+  if (isTRUE(any(grepl("DATA COHORT:", x.upp)))) { data.coh <- .extract.section("DATA COHORT:", x, section.pos) }
 
   #...................
   ### VARIABLE ####
 
-  if (isTRUE(any(grepl("VARIABLE:", x)))) { inpvariable <- .extract.section("VARIABLE:", x, section.pos) }
+  if (isTRUE(any(grepl("VARIABLE:", x.upp)))) { inpvariable <- .extract.section("VARIABLE:", x, section.pos) }
 
   #...................
   ### DEFINE ####
 
-  if (isTRUE(any(grepl("DEFINE:", x)))) { define <- .extract.section("DEFINE:", x, section.pos) }
+  if (isTRUE(any(grepl("DEFINE:", x.upp)))) { define <- .extract.section("DEFINE:", x, section.pos) }
 
   #...................
   ### ANALYSIS ####
 
-  if (isTRUE(any(grepl("ANALYSIS:", x)))) { analysis <- .extract.section("ANALYSIS:", x, section.pos) }
+  if (isTRUE(any(grepl("ANALYSIS:", x.upp)))) { analysis <- .extract.section("ANALYSIS:", x, section.pos) }
 
   #...................
   ### MODEL ####
 
-  if (isTRUE(any(grepl("MODEL:", x)))) { model <- .extract.section("MODEL:", x, section.pos) }
+  if (isTRUE(any(grepl("MODEL:", x.upp)))) { model <- .extract.section("MODEL:", x, section.pos) }
 
   #...................
   ### MODEL INDIRECT ####
 
-  if (isTRUE(any(grepl("MODEL INDIRECT:", x)))) { mod.ind <- .extract.section("MODEL INDIRECT:", x, section.pos) }
+  if (isTRUE(any(grepl("MODEL INDIRECT:", x.upp)))) { mod.ind <- .extract.section("MODEL INDIRECT:", x, section.pos) }
 
   #...................
   ### MODEL TEST ####
 
-  if (isTRUE(any(grepl("MODEL TEST:", x)))) { mod.test <- .extract.section("MODEL TEST:", x, section.pos) }
+  if (isTRUE(any(grepl("MODEL TEST:", x.upp)))) { mod.test <- .extract.section("MODEL TEST:", x, section.pos) }
 
   #...................
   ### MODEL PRIORS ####
 
-  if (isTRUE(any(grepl("MODEL PRIORS:", x)))) { mod.prior <- .extract.section("MODEL PRIORS:", x, section.pos) }
+  if (isTRUE(any(grepl("MODEL PRIORS:", x.upp)))) { mod.prior <- .extract.section("MODEL PRIORS:", x, section.pos) }
 
   #...................
   ### MONTECARLO ####
 
-  if (isTRUE(any(grepl("MONTECARLO:", x)))) { montecarlo <- .extract.section("MONTECARLO:", x, section.pos) }
+  if (isTRUE(any(grepl("MONTECARLO:", x.upp)))) { montecarlo <- .extract.section("MONTECARLO:", x, section.pos) }
 
   #...................
   ### MODEL POPULATION ####
 
-  if (isTRUE(any(grepl("MODEL POPULATION:", x)))) { mod.pop <- .extract.section("MODEL POPULATION:", x, section.pos) }
+  if (isTRUE(any(grepl("MODEL POPULATION:", x.upp)))) { mod.pop <- .extract.section("MODEL POPULATION:", x, section.pos) }
 
   #...................
   ### MODEL COVERAGE ####
 
-  if (isTRUE(any(grepl("MODEL COVERAGE:", x)))) { mod.cov <- .extract.section("MODEL COVERAGE:", x, section.pos) }
+  if (isTRUE(any(grepl("MODEL COVERAGE:", x.upp)))) { mod.cov <- .extract.section("MODEL COVERAGE:", x, section.pos) }
 
   #...................
   ### MODEL MISSING ####
 
-  if (isTRUE(any(grepl("MODEL MISSING:", x)))) { mod.miss <- .extract.section("MODEL MISSING:", x, section.pos) }
+  if (isTRUE(any(grepl("MODEL MISSING:", x.upp)))) { mod.miss <- .extract.section("MODEL MISSING:", x, section.pos) }
 
   #...................
-  ### OUTPUT ####
+  ### MODEL CONSTRAINT ####
 
-  if (isTRUE(any(grepl("OUTPUT:", x)))) { inpoutput <- .extract.section("OUTPUT:", x, section.pos) }
-
-  #...................
-  ### SAVEDATA ####
-
-  if (isTRUE(any(grepl("SAVEDATA:", x)))) { savedata <- .extract.section("SAVEDATA:", x, section.pos) }
+  if (isTRUE(any(grepl("MODEL CONSTRAINT:", x.upp)))) { mod.const <- .extract.section("MODEL CONSTRAINT:", x, section.pos) }
 
   #...................
   ### PLOT ####
 
-  if (isTRUE(any(grepl("PLOT:", x)))) { plot <- .extract.section("PLOT:", x, section.pos) }
+  if (isTRUE(any(grepl("PLOT:", x.upp)))) { plot <- .extract.section("PLOT:", x, section.pos) }
 
-  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  ## Result Object ####
+  #...................
+  ### OUTPUT ####
 
-  input.object <- list(title = title,
-                       data = inpdata, data.imp = data.imp, dat.wl = dat.wl, data.lw = data.lw, data.tp = data.tp, data.miss = data.miss, data.surv = data.surv, data.coh = data.coh,
-                       variable = inpvariable, define = define, analysis = analysis,
-                       model = model, mod.ind = mod.ind, mod.test = mod.test, mod.prior = mod.prior,
-                       montecarlo = montecarlo, mod.pop = mod.pop, mod.cov = mod.cov, mod.miss = mod.miss,
-                       output = inpoutput, savedata = savedata, plot = plot)
+  if (isTRUE(any(grepl("OUTPUT:", x.upp)))) { inpoutput <- .extract.section("OUTPUT:", x, section.pos) }
 
-  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  ## Write Object ####
+  #...................
+  ### SAVEDATA ####
+
+  if (isTRUE(any(grepl("SAVEDATA:", x.upp)))) { savedata <- .extract.section("SAVEDATA:", x, section.pos) }
 
-  write.object <- paste(unlist(Filter(Negate(is.null), input.object)), collapse = "\n\n")
 
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## Variable Names ####
 
-  if (isTRUE(grepl("...", write.object, fixed = TRUE))) {
+  if (isTRUE(grepl("...;", inpvariable, fixed = TRUE))) {
 
     ### Prepare Variable Names ####
-    names.are <- names.temp <- names.length <- "        "
+    names.are <- names.temp <- names.length <- paste0(paste0(rep(" ", times = n.space <- as.numeric(gregexec("\\.\\.\\.;", inpvariable)[[1L]]) - (as.numeric(gregexec("VARIABLE:", inpvariable)[[1L]]) + 10L)), collapse = ""), "        ", collapse = "")
     for (i in colnames(data)) {
 
+      # Replace "." with "_"
+      if (isTRUE(grepl("\\.", i))) { i <- gsub("\\.", "_", i) }
+
       names.temp <- paste(names.are, i, collapse = " ")
       names.length <- paste(names.length, i, collapse = " ")
 
@@ -471,11 +520,39 @@ mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL,
 
     }
 
-    # Replace ...
-    write.object <- sub("...", paste("NAMES ARE\n", paste0(names.are, ";"), "\n", collapse = " "), write.object, fixed = TRUE)
+    # Replace ...;
+    inpvariable <- sub("...;", paste("NAMES ARE\n", paste0(names.are, ";"), "\n", collapse = " "), inpvariable, fixed = TRUE)
 
   }
 
+  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+  ## Input Object ####
+
+  input.object <- list(title = title,
+                       data = inpdata, data.imp = data.imp, data.wl = data.wl, data.lw = data.lw,
+                       data.tp = data.tp, data.miss = data.miss, data.surv = data.surv, data.coh = data.coh,
+                       variable = inpvariable, define = define, analysis = analysis,
+                       model = model, mod.ind = mod.ind, mod.test = mod.test, mod.prior = mod.prior,
+                       montecarlo = montecarlo, mod.pop = mod.pop, mod.cov = mod.cov, mod.miss = mod.miss,
+                       mod.const = mod.const, plot = plot, output = inpoutput, savedata = savedata)
+
+  #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+  ## Write Object ####
+
+  write.object <- paste(sapply(unlist(Filter(Negate(is.null), input.object)), function(y) {
+
+    if (substr(y, start = nchar(y), stop = nchar(y)) != "\n") {
+
+      return(paste0(y, "\n"))
+
+    } else {
+
+      return(y)
+
+    }
+
+  }), collapse = "\n")
+
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## Write Input ####
 
@@ -514,7 +591,7 @@ mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL,
 
       cat("Running Model:", file, "\n")
 
-      misty::mplus.run(file, recursive = FALSE, Mplus = "Mplus", replace.out = replace.out, message = FALSE, show.out = show.out)
+      misty::mplus.run(file, recursive = FALSE, Mplus = Mplus, replace.out = replace.out, message = FALSE, show.out = show.out)
 
       if (isTRUE(output)) { cat("\n") }
 
@@ -535,7 +612,7 @@ mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL,
     # Existing output file
     if (isTRUE(file.exists(file.out))) {
 
-      result.object <- misty::mplus.print(file.out, print = print, input = input, result = result,
+      result.object <- misty::mplus.print(tolower(file.out), print = print, input = input, result = result,
                                           exclude = exclude, variable = variable, not.input = not.input,
                                           not.result = not.result, write = write, append = append, check = FALSE)
 
@@ -563,7 +640,7 @@ mplus <- function(x, file = "Mplus_Input.inp", replace.inp = TRUE, data = NULL,
                  x = x,
                  args = list(file = file, replace.inp = replace.inp, data = data, mplus.run = mplus.run,
                              show.out = show.out, replace.out = replace.out,
-                             print = print, input = input, result = result, exclude = exclude,
+                             Mplus = Mplus, print = print, input = input, result = result, exclude = exclude,
                              variable = variable, not.input = not.input, not.result = not.result,
                              write = write, append = append, check = check, output = output),
                  input = input.object, write = write.object,
