@@ -2,11 +2,8 @@
 #'
 #' This function prints the input command sections and the result sections of a
 #' Mplus output file (\code{.out}) on the R console. By default, the function
-#' prints (1) the input command section excluding the \code{TITLE} section, the
-#' \code{OUTPUT} section, and the names of the variables in the data set
-#' (\code{NAMES} option), and (2) selected result sections, e.g., short
-#' \code{Summary of Analysis}, short \code{Summary of Data}, \code{Model Fit Information},
-#' and \code{Model Results}.
+#' prints selected result sections, e.g., short \code{Summary of Analysis}, short
+#' \code{Summary of Data}, \code{Model Fit Information}, and \code{Model Results}.
 #'
 #' @param x          a character string indicating the name of the Mplus output
 #'                   file with or without the file extension \code{.out}, e.g.,
@@ -15,13 +12,13 @@
 #'                   can be specified, i.e., result object of the \code{mplus.print()},
 #'                   \code{mplus()} or \code{mplus.update()} function.
 #' @param print      a character vector indicating which results to show, i.e.
-#'                   \code{"all"} (default) for all results \code{"input"} for
-#'                   input command sections, and \code{"result"} for result sections.
-#' @param input      a character vector specifiying Mplus input command sections
+#'                   \code{"all"}  for all results \code{"input"} for input command
+#'                   sections, and \code{"result"} (default) for result sections.
+#' @param input      a character vector specifying Mplus input command sections
 #                    included in the output (see 'Details').
-#' @param result     a character vector specifiying Mplus result sections included
+#' @param result     a character vector specifying Mplus result sections included
 #'                   in the output (see 'Details').
-#' @param exclude    a character vector specifiying Mplus input command or result
+#' @param exclude    a character vector specifying Mplus input command or result
 #'                   sections excluded from the output (see 'Details').
 #' @param variable   logical: if \code{TRUE}, names of the variables in the data
 #'                   set (\code{NAMES} option) specified in the \code{VARIABLE:}
@@ -63,7 +60,7 @@
 #'       rearrange a univariate long format to a multivariate wide format.
 #'       \item{\code{"data.tp"}} for the \code{DATA TWOPART} command used to create
 #'       a binary and a continuous variable from a continuous variable with a floor
-#'       effect for use in two-part moding.
+#'       effect for use in two-part modeling.
 #'       \item{\code{"data.miss"}} for the \code{DATA MISSING} command used to
 #'       create a set of binary variables that are indicators of missing data or
 #'       dropout for another set of variables.
@@ -79,10 +76,10 @@
 #'       existing variables and to create new variables.
 #'       \item{\code{"analysis"}} for the \code{ANALYSIS} command used to describe
 #'       the technical details for the analysis.
-#'       \item{\code{"model"}} \code{MODEL} for the  command used to destribe the
+#'       \item{\code{"model"}} \code{MODEL} for the  command used to describe the
 #'       model to be estimated.
 #'       \item{\code{"mod.ind"}} for the \code{MODEL INDIRECT} command used to
-#'       request indirect and directd effects and their standard errors.
+#'       request indirect and directed effects and their standard errors.
 #'       \item{\code{"mod.test"}} for the \code{MODEL TEST} command used to
 #'       test restrictions on the parameters in the \code{MODEL} and \code{MODEL CONSTRAINT}
 #'       commands using the Wald chi-square test.
@@ -92,7 +89,7 @@
 #'       \item{\code{"montecarlo"}} for the \code{MONTECARLO} command used to set
 #'       up and carry out a Monte Carlo simulation study.
 #'       \item{\code{"mod.pop"}} for the \code{MODEL POPULATION} command used
-#'       to provivde the population parameter values to be used in data generation
+#'       to provide the population parameter values to be used in data generation
 #'       using the options of the \code{MODEL} command.
 #'       \item{\code{"mod.cov"}} for the \code{MODEL COVERAGE} used to provide
 #'       the population parameter values to be used for computing coverage.
@@ -106,7 +103,7 @@
 #'       future use.
 #'       \item{\code{"plot"}} for the \code{PLOT} command used to requested graphical
 #'       displays of observed data and analysis results.
-#'       \item{\code{"message"}} for warning and error messsages that have been
+#'       \item{\code{"message"}} for warning and error messages that have been
 #'       generated by the program after the input command sections.
 #'    }
 #' Note that all input command sections are requested by specifying \code{input = "all"}.
@@ -117,11 +114,11 @@
 #' to exclude input command sections from the output (e.g., exclude = "variable").
 #' }
 #' \item{\strong{Result Sections}}{Following result sections can be selected by
-#' using the \code{input} argument or excluded by using the \code{exclude}
+#' using the \code{result} argument or excluded by using the \code{exclude}
 #' argument:
 #'    \itemize{
 #'       \item{\code{"summary.analysis"}} for the \code{SUMMARY OF ANALYSIS} section..
-#'       \item{\code{"summary.analysis.short"}} for a short \code{SUMMARY OF ANALYSIS} section including the number of observations, nuber of groups, estimator, and optimization algorithm.
+#'       \item{\code{"summary.analysis.short"}} for a short \code{SUMMARY OF ANALYSIS} section including the number of observations, number of groups, estimator, and optimization algorithm.
 #'       \item{\code{"summary.data"}} for the \code{SUMMARY OF DATA} section indicating.
 #'       \item{\code{"summary.data.short"}} for a short \code{SUMMARY OF DATA} section including number of clusters, average cluster size, and estimated intraclass correlations.
 #'       \item{\code{"prop.count"}} for the \code{UNIVARIATE PROPORTIONS AND COUNTS FOR CATEGORICAL VARIABLES} section.
@@ -198,7 +195,8 @@
 #'
 #' @seealso
 #' \code{\link{read.mplus}}, \code{\link{write.mplus}}, \code{\link{mplus}},
-#' \code{\link{mplus.update}}, \code{\link{mplus.run}}, \code{\link{mplus.lca}}
+#' \code{\link{mplus.update}}, \code{\link{mplus.plot}}, \code{\link{mplus.bayes}},
+#' \code{\link{mplus.run}}, \code{\link{mplus.lca}}
 #'
 #' @references
 #' Muthen, L. K., & Muthen, B. O. (1998-2017). \emph{Mplus User's Guide} (8th ed.).
@@ -210,7 +208,7 @@
 #'
 #' \item{\code{call}}{function call}
 #' \item{\code{type}}{type of analysis}
-#' \item{\code{x}}{chracter string or misty object}
+#' \item{\code{x}}{character string or misty object}
 #' \item{\code{args}}{specification of function arguments}
 #' \item{\code{print}}{print objects}
 #' \item{\code{notprint}}{character vectors indicating the input commands and
@@ -282,7 +280,7 @@ mplus.print <- function(x, print = c("all", "input", "result"),
   if (isTRUE(is.null(x))) { stop("Input specified for the argument 'x' is NULL.", call. = FALSE) }
 
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  ## Chracter string ####
+  ## Chraacter string ####
 
   if (isTRUE(is.character(x))) {
 
@@ -310,7 +308,7 @@ mplus.print <- function(x, print = c("all", "input", "result"),
 
   #_____________________________________________________________________________
   #
-  # Arguments ------------------------------------------------------------------
+  # Input and Result Arguments -------------------------------------------------
 
   # All input options
   input.all <- c("title", "data", "data.imp", "data.wl", "data.lw", "data.tp", "data.miss", "data.surv", "data.coh", "variable", "define", "analysis", "model", "mod.ind", "mod.test", "mod.prior", "montecarlo", "mod.pop", "mod.cov", "mod.miss", "inpoutput", "savedata", "plot", "output", "message")
@@ -369,7 +367,17 @@ mplus.print <- function(x, print = c("all", "input", "result"),
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## 'print' Argument ####
 
-  if (isTRUE(all(c("all", "input", "result") %in% print) || all(print == "all"))) { print <- c("input", "result") }
+  # Default
+  if (isTRUE(all(c("all", "input", "result") %in% print) || all(print == "all"))) {
+
+    print <- "result"
+
+  # All, input and results
+  } else if (isTRUE(any(print == "all"))) {
+
+    print <- c("input", "result")
+
+  }
 
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## 'input' Argument ####
@@ -390,7 +398,7 @@ mplus.print <- function(x, print = c("all", "input", "result"),
     input <- input.all[input.all %in% misty::chr.omit(union(c("data", "variable", "define", "analysis", "model", "montecarlo", "mod.pop", "mod.cov", "mod.miss", "message"), input), "default", check = FALSE)]
 
   # Manual default setting
-  } else if (isTRUE("default" %in% input && length(input == 1L))) {
+  } else if (isTRUE(all(input == "default"))) {
 
     input <- c("data", "variable", "define", "analysis", "model", "montecarlo", "mod.pop", "mod.cov", "mod.miss", "message")
 
@@ -399,11 +407,15 @@ mplus.print <- function(x, print = c("all", "input", "result"),
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## 'result' Argument ####
 
+  result.default <- FALSE
+
   # Default setting
   if (isTRUE(all(c("all", "default", "summary.analysis.short", "summary.data.short", "random.starts", "summary.fit", "mod.est", "fit", "class.count", "classif", "mod.result", "total.indirect") %in% result))) {
 
     result <- result[!result %in% c("all", "default")]
 
+    result.default <- TRUE
+
   # All result sections
   } else if (isTRUE("all" %in% result)) {
 
@@ -452,6 +464,19 @@ mplus.print <- function(x, print = c("all", "input", "result"),
 
     if (isTRUE(all(!misty::chr.grepl(c("Mplus VERSION", "Mplus DEVELOPMENT"), out)))) { stop("Output file specified in the argument 'x' is not a Mplus output file.", call. = FALSE) }
 
+    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+    ## Default Setting for the Result Argument ####
+
+    if (isTRUE(result.default && any(out == "Chi-Square Test of Model Fit"))) {
+
+      if (isTRUE(as.numeric(misty::chr.trim(sub("Degrees of Freedom", "", out[which(out == "Chi-Square Test of Model Fit") + 3L]))) == 0L)) {
+
+        result <- misty::chr.omit(result, omit = "fit")
+
+      }
+
+    }
+
     #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     ## Exclude Output ####
 
@@ -1306,7 +1331,7 @@ mplus.print <- function(x, print = c("all", "input", "result"),
                                        savedata = if (!is.null(savedata)) { run.val[savedata] } else { NULL },
                                        plot = if (!is.null(plot)) { run.val[plot] } else { NULL },
                                        message = if (!is.null(message)) { run.val[message] } else { NULL }),
-                                       # Output sections
+                          # Output sections
                           result = list(summary.analysis = if (!is.null(summary.analysis)) { run.val[summary.analysis] } else { NULL },
                                         summary.analysis.short = if (!is.null(summary.analysis.short)) { run.val[summary.analysis.short] } else { NULL },
                                         summary.data = if (!is.null(summary.data)) { run.val[summary.data] } else { NULL },
@@ -1396,7 +1421,7 @@ mplus.print <- function(x, print = c("all", "input", "result"),
 
   #_____________________________________________________________________________
   #
-  # Print Objects --------------------------------------------------------------
+  # Print Object ---------------------------------------------------------------
 
   #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   ## Print Input ####
@@ -1543,6 +1568,16 @@ mplus.print <- function(x, print = c("all", "input", "result"),
 
   if (isTRUE(output)) {
 
+    #...................
+    ### ERROR in Output ####
+
+    if (isTRUE(any(grepl("\\*\\*\\* ERROR", return.object$input$message)))) {
+
+      print <- "input"
+      not.result <- not.input <- FALSE
+
+    }
+
     #...................
     ### Print Input Commands ####
 
